@@ -14,17 +14,18 @@ from .error import InvalidEffectException
 from .section import Section, SectionAdapter
 from .device import DeviceAdapter
 from . import effects
+from . import colors
 from .history import History
 from .effect import Effect
+from .color import ColorType
 
 __author__ = "Nick Pesce"
 __email__ = "nickpesce22@gmail.com"
 
-FPS = 40
-SPEED = 0b10
-COLOR = 0b1
-MAX_SHOW_SPEED = 60  # hertz
+# If the system is running slow, this many effect updates can happen before each light update
+CATCHUP_UPDATES = 3
 TIME_WARN_COOLDOWN = 10  # seconds
+TIME_WARN_THRESHOLD = 0.1  # seconds
 logger = logging.getLogger(__name__)
 
 
@@ -35,6 +36,7 @@ class commands:
         logger.info("Using config directory: %s", self.config_path)
         self.sleep_event = threading.Event()
 
+        self.color_types = {}
         self.effects = {}
         self.effects_by_id = {}
         self.transactions = {}
@@ -45,14 +47,11 @@ class commands:
         self.history = History(self)
 
         # Load config files
-        with open(os.path.join(self.config_path, "speeds.json")) as data_file:
-            self.speeds = json.load(data_file)
-
         with open(os.path.join(self.config_path, "presets.json")) as data_file:
             self.presets = json.load(data_file)
 
         with open(os.path.join(self.config_path, "colors.json")) as data_file:
-            self.colors = json.load(data_file)
+            self.named_rgb_colors = json.load(data_file)
 
         with open(os.path.join(self.config_path, "ranges.json")) as data_file:
             range_json = json.load(data_file)
@@ -110,6 +109,7 @@ class commands:
                 self.zones[zone["name"]] = zone["sections"]
 
         self.controller_manager = ControllerManager(self.sections)
+        self.import_colors()
         self.import_effects()
         atexit.register(self._clean_shutdown)
         self.start_loop()
@@ -128,7 +128,11 @@ class commands:
                 try:
                     with self.show_lock:
                         for effect in self.effects_by_id.values():
-                            if effect.next_upd_time <= start_time:
+                            updates = 0
+                            while (
+                                effect.next_upd_time <= start_time
+                                and updates < CATCHUP_UPDATES
+                            ):
                                 start_upd = time.time()
                                 try:
                                     effect.update()
@@ -142,22 +146,25 @@ class commands:
                                     (end_upd - start_upd) * 1000,
                                     str(effect),
                                 )
-                                next_upd_time = min(next_upd_time, effect.next_upd_time)
-
+                                updates += 1
+                            next_upd_time = min(next_upd_time, effect.next_upd_time)
+                        logger.debug(
+                            "Took %dms to do full update",
+                            (time.time() - start_time) * 1000,
+                        )
+                        self.controller_manager.show()
                     if next_upd_time == float("inf"):
+                        logger.debug("No effects updated")
                         continue
+
                     wait_time = next_upd_time - time.time()
-                    if wait_time < 0:
+                    if wait_time < -1 * TIME_WARN_THRESHOLD:
                         if last_warn_time < time.time() - TIME_WARN_COOLDOWN:
                             logger.warning(
                                 "Can't keep up! Did the system time change, "
                                 "or is the server overloaded? Running %d ms behind.",
                                 (wait_time * -1000),
                             )
-                            logger.debug(
-                                "Behind while running effect(s) %s",
-                                str(self.effects_by_id.values()),
-                            )
                             last_warn_time = time.time()
                     else:
                         self.sleep_event.wait(wait_time)
@@ -167,24 +174,6 @@ class commands:
                         self.show_lock.release()
                     logger.exception("Error in effect loop")
 
-        def show_loop(self):
-            start_time = time.time()
-            while self.should_run:
-                try:
-                    self.controller_manager.show(self.show_lock)
-                    end_time = time.time()
-                    show_time = end_time - start_time
-                    logger.debug("Show took %dms", show_time * 1000)
-                    wait_time = max(0, (1 / MAX_SHOW_SPEED) - show_time)
-                    self.sleep_event.wait(wait_time)
-                    start_time = end_time
-                except Exception:
-                    logger.exception("Error in show loop")
-                    if self.show_lock.locked():
-                        self.show_lock.release()
-
-        self.show_thread = threading.Thread(target=show_loop, args=(self,))
-        self.show_thread.start()
         self.loop_thread = threading.Thread(target=loop, args=(self,))
         self.loop_thread.start()
 
@@ -192,7 +181,6 @@ class commands:
         self.sleep_event.set()
         self.should_run = False
         self.loop_thread.join()
-        self.show_thread.join()
 
     def start_preset(self, preset_name, properties, transaction_id):
         preset = self.presets.get(preset_name, None)
@@ -236,41 +224,46 @@ class commands:
         if not self.is_effect(effect_name):
             return None
         effect = self.effects[effect_name.lower()]
-        # remove any 'None' args
-        args = {k: v for (k, v) in args.items() if v is not None}
-        # attempt to parse arg values
-        args = {k: self.get_value_from_string(k, args[k]) for k in args}
         sections = self.get_sections_from_ranges(
             properties.get("ranges", [self.default_range])
         )
-        self.show_lock.acquire()
-        opacity = properties.get("opacity", 1)
-        if not isinstance(opacity, (int, float)):
-            logger.warning("Invalid 'opacity' property received: %s", str(opacity))
-            opacity = 1
-        overlayed = properties.get("overlayed", False)
-        if not isinstance(overlayed, bool):
-            logger.warning("Invalid 'overlayed' property received: %s", str(overlayed))
-            overlayed = False
-        controller = self.controller_manager.create_controller(
-            sections, overlayed=overlayed, opacity=opacity
-        )
-        self.effects_by_id = {
-            effect_id: effect
-            for effect_id, effect in self.effects_by_id.items()
-            if effect.controller.size > 0
-        }
+        with self.show_lock:
+            opacity = properties.get("opacity")
+            if opacity is None:
+                opacity = 1
+            if not isinstance(opacity, (int, float)):
+                logger.warning("Invalid 'opacity' property received: %s", str(opacity))
+                opacity = 1
+            overlayed = properties.get("overlayed")
+            if overlayed is None:
+                overlayed = False
+            if not isinstance(overlayed, bool):
+                logger.warning(
+                    "Invalid 'overlayed' property received: %s", str(overlayed)
+                )
+                overlayed = False
+            controller = self.controller_manager.create_controller(
+                sections, overlayed=overlayed, opacity=opacity
+            )
+            self.effects_by_id = {
+                effect_id: effect
+                for effect_id, effect in self.effects_by_id.items()
+                if effect.controller.size > 0
+            }
 
-        # fill in default args from schema
-        schema = effect.schema
-        self.complete_args_with_schema(args, schema, controller)
+            schema = effect.schema
+            # remove any 'None' args
+            args = {k: v for (k, v) in args.items() if v is not None}
+            # fill in default args from schema
+            self.complete_args_with_schema(args, schema, controller)
+            # attempt to parse arg values
+            args = {k: self.process_arg_value(k, args[k], controller) for k in args}
 
-        default_speed = effect.default_speed
-        speed = properties.get("speed", default_speed)
-        effect_instance = effect.create(args, speed, controller, transaction_id)
-        self.transactions.setdefault(transaction_id, []).append(effect_instance.id)
-        self.effects_by_id[effect_instance.id] = effect_instance
-        self.show_lock.release()
+            default_speed = effect.default_speed
+            speed = properties.get("speed", default_speed)
+            effect_instance = effect.create(args, speed, controller, transaction_id)
+            self.transactions.setdefault(transaction_id, []).append(effect_instance.id)
+            self.effects_by_id[effect_instance.id] = effect_instance
         self.sleep_event.set()
         logger.info("New controller manager: %s", str(self.controller_manager))
         return effect_instance
@@ -279,17 +272,19 @@ class commands:
         return self.transactions.get(transaction_id)
 
     def stop_all(self):
-        effect_ids = [effect_id for effect_id in self.effects_by_id]
+        with self.show_lock:
+            effect_ids = [effect_id for effect_id in self.effects_by_id]
         for effect_id in effect_ids:
             self.stop_effect(effect_id)
 
     def stop_effect(self, effect_id):
-        effect = self.effects_by_id.get(effect_id)
-        if effect:
-            self.controller_manager.remove_controller(effect.controller)
-            del self.effects_by_id[effect_id]
-            return True
-        return False
+        with self.show_lock:
+            effect = self.effects_by_id.get(effect_id)
+            if effect:
+                self.controller_manager.remove_controller(effect.controller)
+                del self.effects_by_id[effect_id]
+                return True
+            return False
 
     def complete_args_with_schema(self, args, schema, controller):
         for k, v in sorted(
@@ -320,11 +315,6 @@ class commands:
 
     def schema_to_string(self, schema):
         ret = str(schema)
-        # ret = ""
-        # if(& SPEED):
-        #    ret += "[-s speed]"
-        # if(modifiers & COLOR):
-        #    ret += "[-c (r,g,b)]"
         return ret
 
     def get_effects(self):
@@ -341,10 +331,10 @@ class commands:
         return self.presets
 
     def get_colors(self):
-        return self.colors
+        return self.named_rgb_colors
 
-    def get_speeds(self):
-        return self.speeds
+    def get_color_types(self):
+        return [{"name": ct.name, "schema": ct.schema} for ct in self.color_types.values()]
 
     def get_sections(self):
         return self.sections
@@ -361,20 +351,21 @@ class commands:
         def state_schema(state, schema):
             return {k: v for k, v in state.items() if k in schema}
 
-        for effect in self.effects_by_id.values():
-            controller = effect.controller
-            state.append(
-                {
-                    "sections": controller.active_section_names,
-                    "opacity": controller.opacity,
-                    "effect_name": effect.effect.name,
-                    "effect_state": state_schema(
-                        effect.state, effect.effect.command_schema
-                    ),
-                    "effect_id": effect.id,
-                    "transaction_id": effect.transaction_id,
-                }
-            )
+        with self.show_lock:
+            for effect in self.effects_by_id.values():
+                controller = effect.controller
+                state.append(
+                    {
+                        "sections": controller.active_section_names,
+                        "opacity": controller.opacity,
+                        "effect_name": effect.effect.name,
+                        "effect_state": state_schema(
+                            effect.state, effect.effect.command_schema
+                        ),
+                        "effect_id": effect.id,
+                        "transaction_id": effect.transaction_id,
+                    }
+                )
         return state
 
     def get_sections_from_ranges(self, lst):
@@ -387,30 +378,44 @@ class commands:
                 ret += self.zones[r]
         return ret
 
-    def get_value_from_string(self, type, string):
+    def process_arg_value(self, arg_type, value, lights_controller):
         """Given a attribute represented as a string, convert it to the appropriate value"""
-        if not isinstance(string, str):
-            return string
-        if type.lower() == "color":
-            if string.lower() == "random":
-                return list(
-                    map(
-                        lambda x: int(255 * x),
-                        colorsys.hsv_to_rgb(random.random(), 1, 1),
+        if arg_type.lower() == "color":
+            generator_schema = None
+            rgb = None
+            if isinstance(value, dict):
+                # Color generator
+                generator_schema = value
+            elif isinstance(value, str):
+                if value == "random":
+                    rgb = list(
+                        map(
+                            lambda x: int(255 * x),
+                            colorsys.hsv_to_rgb(random.random(), 1, 1),
+                        )
                     )
+                else:
+                    for color in self.named_rgb_colors:
+                        if color["name"].lower() == value.lower():
+                            rgb = color["rgb"]
+                    raise ValueError(f"Invalid color name: {value}")
+            elif isinstance(value, (tuple, list)):
+                # RGB
+                rgb = list(value)
+            else:
+                raise ValueError(f"Invalid type for color argument: {type(value)}")
+            if not generator_schema:
+                generator_schema = {
+                    "type": "Static Color",
+                    "args": {"color": rgb},
+                }
+            color_type = self.color_types.get(generator_schema["type"].lower())
+            if not color_type:
+                raise ValueError(
+                    f"Invalid color generator type: {generator_schema.get('type')}"
                 )
-            for c in self.colors:
-                if c["name"].lower() == string.lower():
-                    return c["rgb"]
-            # TODO throw invalid color name error
-            return [255, 255, 255]
-        elif type.lower() == "speed":
-            return self.speeds.get(string.lower())
-        elif type.lower() == "ranges":
-            if string.lower() == "all":
-                return self.sections.keys()
-            return string.split(",")
-        return string
+            return color_type.new_generator(generator_schema["args"], lights_controller)
+        return value
 
     def is_effect(self, name):
         return name.lower() in self.effects
@@ -425,7 +430,7 @@ class commands:
             except Exception as e:
                 logger.exception("Could not import %s", m)
 
-        logger.info("Parsing effects")
+        logger.info("Parsing effect modules")
         for module in modules:
             try:
                 logger.info("loading %s", str(module))
@@ -434,12 +439,31 @@ class commands:
             except Exception as e:
                 logger.exception("Error loading effect %s", str(module))
 
+    def import_colors(self):
+        module_names = colors.colors
+        modules = []
+        for m in module_names:
+            try:
+                module = importlib.import_module(m)
+                modules.append(module)
+            except Exception as e:
+                logger.exception("Could not import %s", m)
+
+        logger.info("Parsing color generator modules")
+        for module in modules:
+            try:
+                logger.info("loading %s", str(module))
+                color_type = ColorType(module)
+                self.color_types[color_type.name.lower()] = color_type
+            except Exception as e:
+                logger.exception("Error loading color type %s", str(module))
+
     def _clean_shutdown(self):
         logger.info("Shutting down")
         self.stop_loop()
         for controller in self.controller_manager.get_controllers():
             controller.clear()
-        self.controller_manager.show(self.show_lock)
+        self.controller_manager.show()
 
 
 class Response:
