@@ -7,13 +7,12 @@ description = "Two alternating colors"
 default_speed = 1
 
 schema = {
-    "color 1": {
-        "value": {"type": "color", "default": (255, 0, 0)},
-        "user_input": True,
-        "required": False,
-    },
-    "color 2": {
-        "value": {"type": "color", "default": (0, 0, 255)},
+    "color": {
+        "value": {
+            "type": "color",
+            "default_selector": "palette",
+            "default": (255, 0, 0),
+        },
         "user_input": True,
         "required": False,
     },
@@ -21,8 +20,9 @@ schema = {
 
 
 def update(lights, step, state):
+    colors = state["color"].get_palette(2)
     for i in range(lights.size):
         if (i + step) % 2 == 0:
-            lights.set_pixel(i, *state["color 1"])
+            lights.set_pixel(i, *colors[1])
         else:
-            lights.set_pixel(i, *state["color 2"])
+            lights.set_pixel(i, *colors[2])
