@@ -1,16 +1,10 @@
 name = "Disco"
 
-start_string = name + " started!"
+start_message = name + " started!"
 
 description = "Color spectrum is repeatedly 'squished' to one side."
 
-schema = {
-    "speed": {
-        "value": {"type": "number", "min": 0, "max": 100, "default": 50},
-        "user_input": True,
-        "required": False,
-    }
-}
+schema = {}
 
 
 def update(lights, step, state):
